@@ -19,7 +19,7 @@ from calibre.constants import (iswindows, filesystem_encoding,
         preferred_encoding)
 from calibre.ptempfile import PersistentTemporaryFile, TemporaryFile
 from calibre.db import SPOOL_SIZE
-from calibre.db.annotations import annot_db_data
+from calibre.db.annotations import annot_db_data, unicode_normalize
 from calibre.db.schema_upgrades import SchemaUpgrade
 from calibre.db.delete_service import delete_service
 from calibre.db.errors import NoSuchFormat
@@ -420,7 +420,7 @@ class DB(object):
 
         if iswindows and len(self.library_path) + 4*self.PATH_LIMIT + 10 > 259:
             raise ValueError(_(
-                'Path to library ({0}) too long. Must be less than'
+                'Path to library ({0}) too long. It must be less than'
                 ' {1} characters.').format(self.library_path, 259-4*self.PATH_LIMIT-10))
         exists = self._exists = os.path.exists(self.dbpath)
         if not exists:
@@ -428,7 +428,7 @@ class DB(object):
             # allowed for max path lengths of 265 chars.
             if (iswindows and len(self.library_path) > self.WINDOWS_LIBRARY_PATH_LIMIT):
                 raise ValueError(_(
-                    'Path to library too long. Must be less than'
+                    'Path to library too long. It must be less than'
                     ' %d characters.')%self.WINDOWS_LIBRARY_PATH_LIMIT)
 
         if read_only and os.path.exists(self.dbpath):
@@ -491,7 +491,7 @@ class DB(object):
 
     def set_user_template_functions(self, user_formatter_functions):
         self._user_template_functions = user_formatter_functions
-        self._template_functions = formatter_functions().get_builtins().copy()
+        self._template_functions = formatter_functions().get_builtins_and_aliases().copy()
         self._template_functions.update(user_formatter_functions)
 
     def initialize_prefs(self, default_prefs, restore_all_prefs, progress_callback):  # {{{
@@ -1338,7 +1338,12 @@ class DB(object):
         except:  # If path contains strange characters this throws an exc
             candidates = []
         if fmt and candidates and os.path.exists(candidates[0]):
-            shutil.copyfile(candidates[0], fmt_path)
+            try:
+                shutil.copyfile(candidates[0], fmt_path)
+            except shutil.SameFileError:
+                # some other process synced in the file since the last
+                # os.path.exists()
+                return candidates[0]
             return fmt_path
 
     def cover_abspath(self, book_id, path):
@@ -1469,6 +1474,21 @@ class DB(object):
         with f:
             return True, f.read(), stat.st_mtime
 
+    def compress_covers(self, path_map, jpeg_quality, progress_callback):
+        cpath_map = {}
+        if not progress_callback:
+            progress_callback = lambda book_id, old_sz, new_sz: None
+        for book_id, path in path_map.items():
+            path = os.path.abspath(os.path.join(self.library_path, path, 'cover.jpg'))
+            try:
+                sz = os.path.getsize(path)
+            except OSError:
+                progress_callback(book_id, 0, 'ENOENT')
+            else:
+                cpath_map[book_id] = (path, sz)
+        from calibre.db.covers import compress_covers
+        compress_covers(cpath_map, jpeg_quality, progress_callback)
+
     def set_cover(self, book_id, path, data, no_processing=False):
         path = os.path.abspath(os.path.join(self.library_path, path))
         if not os.path.exists(path):
@@ -1781,6 +1801,7 @@ class DB(object):
         fts_engine_query, use_stemming, highlight_start, highlight_end, snippet_size, annotation_type,
         restrict_to_book_ids, restrict_to_user, ignore_removed=False
     ):
+        fts_engine_query = unicode_normalize(fts_engine_query)
         fts_table = 'annotations_fts_stemmed' if use_stemming else 'annotations_fts'
         text = 'annotations.searchable_text'
         if highlight_start is not None and highlight_end is not None:
@@ -1797,7 +1818,7 @@ class DB(object):
         data = [fts_engine_query]
         if restrict_to_user:
             query += ' AND annotations.user_type = ? AND annotations.user = ?'
-            data += list(*restrict_to_user)
+            data += list(restrict_to_user)
         if annotation_type:
             query += ' AND annotations.annot_type = ? '
             data.append(annotation_type)
@@ -1805,6 +1826,8 @@ class DB(object):
         ls = json.loads
         try:
             for (rowid, book_id, fmt, user_type, user, annot_data, text) in self.execute(query, tuple(data)):
+                if restrict_to_book_ids is not None and book_id not in restrict_to_book_ids:
+                    continue
                 try:
                     parsed_annot = ls(annot_data)
                 except Exception:
@@ -1825,7 +1848,7 @@ class DB(object):
 
     def all_annotations_for_book(self, book_id, ignore_removed=False):
         for (fmt, user_type, user, data) in self.execute(
-            'SELECT id, book, format, user_type, user, annot_data FROM annotations WHERE book=?', (book_id,)
+            'SELECT format, user_type, user, annot_data FROM annotations WHERE book=?', (book_id,)
         ):
             try:
                 annot = json.loads(data)
@@ -1875,7 +1898,7 @@ class DB(object):
                     self.execute('UPDATE annotations SET annot_data=?, timestamp=?, annot_type=?, searchable_text=?, annot_id=? WHERE id=?',
                         (json.dumps(annot), timestamp, atype, text, aid, annot_id))
 
-    def all_annotations(self, restrict_to_user=None, limit=None, annotation_type=None, ignore_removed=False):
+    def all_annotations(self, restrict_to_user=None, limit=None, annotation_type=None, ignore_removed=False, restrict_to_book_ids=None):
         ls = json.loads
         q = 'SELECT id, book, format, user_type, user, annot_data FROM annotations'
         data = []
@@ -1891,6 +1914,8 @@ class DB(object):
         q += ' ORDER BY timestamp DESC '
         count = 0
         for (rowid, book_id, fmt, user_type, user, annot_data) in self.execute(q, tuple(data)):
+            if restrict_to_book_ids is not None and book_id not in restrict_to_book_ids:
+                continue
             try:
                 annot = ls(annot_data)
                 atype = annot['type']
@@ -1944,6 +1969,14 @@ class DB(object):
                 self.execute('DELETE FROM annotations_dirtied')
         return changed
 
+    def annotation_count_for_book(self, book_id):
+        for (count,) in self.execute('''
+                 SELECT count(id) FROM annotations
+                 WHERE book=? AND json_extract(annot_data, "$.removed") IS NULL
+                 ''', (book_id,)):
+            return count
+        return 0
+
     def conversion_options(self, book_id, fmt):
         for (data,) in self.conn.get('SELECT data FROM conversion_options WHERE book=? AND format=?', (book_id, fmt.upper())):
             if data:
