@@ -1,32 +1,31 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import time, re
+import re
+import time
 from datetime import datetime
-from Queue import Queue, Empty
-from threading import Thread
-from io import BytesIO
+from io import StringIO
 from operator import attrgetter
-from urlparse import urlparse
-from urllib import quote
+from threading import Thread
 
-from calibre.customize.ui import metadata_plugins, all_metadata_plugins
-from calibre.ebooks.metadata import check_issn
+from calibre.customize.ui import all_metadata_plugins, metadata_plugins
+from calibre.ebooks.metadata import authors_to_sort_string, check_issn
+from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.sources.base import create_log
 from calibre.ebooks.metadata.sources.prefs import msprefs
 from calibre.ebooks.metadata.xisbn import xisbn
-from calibre.ebooks.metadata.book.base import Metadata
-from calibre.utils.date import utc_tz, as_utc
-from calibre.utils.html2text import html2text
-from calibre.utils.icu import lower
-from calibre.utils.date import UNDEFINED_DATE
+from calibre.utils.date import UNDEFINED_DATE, as_utc, utc_tz
 from calibre.utils.formatter import EvalFormatter
+from calibre.utils.html2text import html2text
+from calibre.utils.icu import lower, primary_sort_key
+from polyglot.builtins import iteritems, itervalues, unicode_type
+from polyglot.queue import Empty, Queue
+from polyglot.urllib import quote, urlparse
 
 # Download worker {{{
 
@@ -39,7 +38,7 @@ class Worker(Thread):
 
         self.plugin, self.kwargs, self.rq = plugin, kwargs, Queue()
         self.abort = abort
-        self.buf = BytesIO()
+        self.buf = StringIO()
         self.log = create_log(self.buf)
 
     def run(self):
@@ -92,11 +91,13 @@ class ISBNMerge(object):
         self.isbnless_results = []
         self.results = []
         self.log = log
-        self.use_xisbn = True
+        # The xISBN service has been de-commissioned
+        # https://www.oclc.org/developer/news/2018/xid-decommission.en.html
+        self.use_xisbn = False
 
     def isbn_in_pool(self, isbn):
         if isbn:
-            for isbns, pool in self.pools.iteritems():
+            for isbns, pool in iteritems(self.pools):
                 if isbn in isbns:
                     return pool
         return None
@@ -144,7 +145,7 @@ class ISBNMerge(object):
 
     def finalize(self):
         has_isbn_result = False
-        for results in self.pools.itervalues():
+        for results in itervalues(self.pools):
             if results:
                 has_isbn_result = True
                 break
@@ -163,7 +164,7 @@ class ISBNMerge(object):
             # Pick only the most relevant result from each source
             seen = set()
             for result in results:
-                if result.identify_plugin not in seen:
+                if msprefs['keep_dups'] or result.identify_plugin not in seen:
                     seen.add(result.identify_plugin)
                     self.results.append(result)
                     result.average_source_relevance = \
@@ -182,14 +183,14 @@ class ISBNMerge(object):
         groups = {}
         for result in self.results:
             title = lower(result.title if result.title else '')
-            key = (title, tuple([lower(x) for x in result.authors]))
+            key = (title, tuple(lower(x) for x in result.authors))
             if key not in groups:
                 groups[key] = []
             groups[key].append(result)
 
         if len(groups) != len(self.results):
             self.results = []
-            for rgroup in groups.itervalues():
+            for rgroup in itervalues(groups):
                 rel = [r.average_source_relevance for r in rgroup]
                 if len(rgroup) > 1:
                     result = self.merge(rgroup, None, do_asr=False)
@@ -203,7 +204,7 @@ class ISBNMerge(object):
             groups, empty = {}, []
             for result in self.results:
                 key = set()
-                for typ, val in result.identifiers.iteritems():
+                for typ, val in iteritems(result.identifiers):
                     if typ and val:
                         key.add((typ, val))
                 if key:
@@ -224,7 +225,7 @@ class ISBNMerge(object):
 
             if len(groups) != len(self.results):
                 self.results = []
-                for rgroup in groups.itervalues():
+                for rgroup in itervalues(groups):
                     rel = [r.average_source_relevance for r in rgroup]
                     if len(rgroup) > 1:
                         result = self.merge(rgroup, None, do_asr=False)
@@ -241,7 +242,7 @@ class ISBNMerge(object):
     def merge_isbn_results(self):
         self.results = []
         sources = set()
-        for min_year, results in self.pools.itervalues():
+        for min_year, results in itervalues(self.pools):
             if results:
                 for r in results:
                     sources.add(r.identify_plugin)
@@ -359,7 +360,7 @@ class ISBNMerge(object):
 
 def merge_identify_results(result_map, log):
     isbn_merge = ISBNMerge(log)
-    for plugin, results in result_map.iteritems():
+    for plugin, results in iteritems(result_map):
         for result in results:
             isbn_merge.add_result(result)
 
@@ -423,8 +424,7 @@ def identify(log, abort,  # {{{
         if not is_worker_alive(workers):
             break
 
-        if (first_result_at is not None and time.time() - first_result_at >
-                wait_time):
+        if (first_result_at is not None and time.time() - first_result_at > wait_time):
             log.warn('Not waiting any longer for more results. Still running'
                     ' sources:')
             for worker in workers:
@@ -437,12 +437,12 @@ def identify(log, abort,  # {{{
         pass
 
     sort_kwargs = dict(kwargs)
-    for k in list(sort_kwargs.iterkeys()):
+    for k in list(sort_kwargs):
         if k not in ('title', 'authors', 'identifiers'):
             sort_kwargs.pop(k)
 
     longest, lp = -1, ''
-    for plugin, presults in results.iteritems():
+    for plugin, presults in iteritems(results):
         presults.sort(key=plugin.identify_results_keygen(**sort_kwargs))
 
         # Throw away lower priority results from the same source that have exactly the same
@@ -470,7 +470,7 @@ def identify(log, abort,  # {{{
         for r in presults:
             log('\n\n---')
             try:
-                log(unicode(r))
+                log(unicode_type(r))
             except TypeError:
                 log(repr(r))
         if plog:
@@ -486,8 +486,7 @@ def identify(log, abort,  # {{{
                     result.series_index = dummy.series_index
             result.relevance_in_source = i
             result.has_cached_cover_url = (
-                plugin.cached_cover_url_is_reliable and
-                plugin.get_cached_cover_url(result.identifiers) is not None)
+                plugin.cached_cover_url_is_reliable and plugin.get_cached_cover_url(result.identifiers) is not None)
             result.identify_plugin = plugin
             if msprefs['txt_comments']:
                 if plugin.has_html_comments and result.comments:
@@ -495,8 +494,7 @@ def identify(log, abort,  # {{{
 
     log('The identify phase took %.2f seconds'%(time.time() - start_time))
     log('The longest time (%f) was taken by:'%longest, lp)
-    log('Merging results from different sources and finding earliest ',
-            'publication dates from the worldcat.org service')
+    log('Merging results from different sources')
     start_time = time.time()
     results = merge_identify_results(results, log)
 
@@ -505,6 +503,10 @@ def identify(log, abort,  # {{{
     tm_rules = msprefs['tag_map_rules']
     if tm_rules:
         from calibre.ebooks.metadata.tag_mapper import map_tags
+    am_rules = msprefs['author_map_rules']
+    if am_rules:
+        from calibre.ebooks.metadata.author_mapper import compile_rules, map_authors
+        am_rules = compile_rules(am_rules)
 
     max_tags = msprefs['max_tags']
     for r in results:
@@ -526,18 +528,35 @@ def identify(log, abort,  # {{{
                 return '%s, %s' % (surname, ' '.join(parts[:-1]))
             r.authors = [swap_to_ln_fn(a) for a in r.authors]
 
+    if am_rules:
+        for r in results:
+            new_authors = map_authors(r.authors, am_rules)
+            if new_authors != r.authors:
+                r.authors = new_authors
+                r.author_sort = authors_to_sort_string(r.authors)
+
     return results
 # }}}
 
 
-def urls_from_identifiers(identifiers):  # {{{
-    identifiers = {k.lower():v for k, v in identifiers.iteritems()}
+def urls_from_identifiers(identifiers, sort_results=False):  # {{{
+    identifiers = {k.lower():v for k, v in iteritems(identifiers)}
     ans = []
+    keys_left = set(identifiers)
+
+    def add(name, k, val, url):
+        ans.append((name, k, val, url))
+        keys_left.discard(k)
+
     rules = msprefs['id_link_rules']
     if rules:
         formatter = EvalFormatter()
-        for k, val in identifiers.iteritems():
-            vals = {'id':quote(val if isinstance(val, bytes) else val.encode('utf-8')).decode('ascii')}
+        for k, val in iteritems(identifiers):
+            val = val.replace('|', ',')
+            vals = {
+                'id':unicode_type(quote(val if isinstance(val, bytes) else val.encode('utf-8'))),
+                'id_unquoted': str(val),
+            }
             items = rules.get(k) or ()
             for name, template in items:
                 try:
@@ -546,40 +565,55 @@ def urls_from_identifiers(identifiers):  # {{{
                     import traceback
                     traceback.format_exc()
                     continue
-                ans.append((name, k, val, url))
+                add(name, k, val, url)
     for plugin in all_metadata_plugins():
         try:
             for id_type, id_val, url in plugin.get_book_urls(identifiers):
-                ans.append((plugin.get_book_url_name(id_type, id_val, url), id_type, id_val, url))
-        except:
+                add(plugin.get_book_url_name(id_type, id_val, url), id_type, id_val, url)
+        except Exception:
             pass
     isbn = identifiers.get('isbn', None)
     if isbn:
-        ans.append((isbn, 'isbn', isbn,
-            'https://www.worldcat.org/isbn/'+isbn))
+        add(isbn, 'isbn', isbn,
+            'https://www.worldcat.org/isbn/'+isbn)
     doi = identifiers.get('doi', None)
     if doi:
-        ans.append(('DOI', 'doi', doi,
-            'https://dx.doi.org/'+doi))
+        add('DOI', 'doi', doi,
+            'https://dx.doi.org/'+doi)
     arxiv = identifiers.get('arxiv', None)
     if arxiv:
-        ans.append(('arXiv', 'arxiv', arxiv,
-            'https://arxiv.org/abs/'+arxiv))
+        add('arXiv', 'arxiv', arxiv,
+            'https://arxiv.org/abs/'+arxiv)
     oclc = identifiers.get('oclc', None)
     if oclc:
-        ans.append(('OCLC', 'oclc', oclc,
-            'https://www.worldcat.org/oclc/'+oclc))
+        add('OCLC', 'oclc', oclc,
+            'https://www.worldcat.org/oclc/'+oclc)
     issn = check_issn(identifiers.get('issn', None))
     if issn:
-        ans.append((issn, 'issn', issn,
-            'https://www.worldcat.org/issn/'+issn))
-    for k, url in identifiers.iteritems():
+        add(issn, 'issn', issn,
+            'https://www.worldcat.org/issn/'+issn)
+    q = {'http', 'https', 'file'}
+    for k, url in iteritems(identifiers):
         if url and re.match(r'ur[il]\d*$', k) is not None:
             url = url[:8].replace('|', ':') + url[8:].replace('|', ',')
-            if url.partition(':')[0].lower() in {'http', 'file', 'https'}:
+            if url.partition(':')[0].lower() in q:
                 parts = urlparse(url)
                 name = parts.netloc or parts.path
-                ans.append((name, k, url, url))
+                add(name, k, url, url)
+    for k in tuple(keys_left):
+        val = identifiers.get(k)
+        if val:
+            url = val[:8].replace('|', ':') + val[8:].replace('|', ',')
+            if url.partition(':')[0].lower() in q:
+                parts = urlparse(url)
+                name = parts.netloc or parts.path
+                add(name, k, url, url)
+    if sort_results:
+
+        def url_key(x):
+            return primary_sort_key(str(x[0]))
+
+        ans = sorted(ans, key=url_key)
     return ans
 # }}}
 
@@ -587,8 +621,9 @@ def urls_from_identifiers(identifiers):  # {{{
 if __name__ == '__main__':  # tests {{{
     # To run these test use: calibre-debug -e
     # src/calibre/ebooks/metadata/sources/identify.py
-    from calibre.ebooks.metadata.sources.test import (test_identify,
-            title_test, authors_test)
+    from calibre.ebooks.metadata.sources.test import (
+        authors_test, test_identify, title_test
+    )
     tests = [
             (
                 {'title':'Magykal Papers',
