@@ -1,31 +1,28 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, errno, json, importlib, math, httplib, bz2, shutil, sys
+import os, errno, json, importlib, math, bz2, shutil, sys
 from itertools import count
 from io import BytesIO
-from future_builtins import map
-from Queue import Queue, Empty
 from threading import Thread, Event
 from multiprocessing.pool import ThreadPool
 
-from PyQt5.Qt import (
+from qt.core import (
     QImageReader, QFormLayout, QVBoxLayout, QSplitter, QGroupBox, QListWidget,
     QLineEdit, QSpinBox, QTextEdit, QSize, QListWidgetItem, QIcon, QImage,
-    pyqtSignal, QStackedLayout, QWidget, QLabel, Qt, QComboBox, QPixmap,
-    QGridLayout, QStyledItemDelegate, QModelIndex, QApplication, QStaticText,
-    QStyle, QPen, QProgressDialog
+    pyqtSignal, QStackedLayout, QWidget, QLabel, Qt, QComboBox, QPixmap, QDialog,
+    QGridLayout, QStyledItemDelegate, QApplication, QStaticText, sip,
+    QStyle, QPen, QProgressDialog, QAbstractItemView, QDialogButtonBox
 )
 
 from calibre import walk, fit_image, human_readable, detect_ncpus as cpu_count
 from calibre.constants import cache_dir, config_dir
 from calibre.customize.ui import interface_actions
-from calibre.gui2 import must_use_qt, gprefs, choose_dir, error_dialog, choose_save_file, question_dialog
+from calibre.gui2 import must_use_qt, gprefs, choose_dir, error_dialog, choose_save_file, question_dialog, empty_index
 from calibre.gui2.dialogs.progress import ProgressDialog
 from calibre.gui2.progress_indicator import ProgressIndicator
 from calibre.gui2.widgets2 import Dialog
@@ -36,7 +33,9 @@ from calibre.utils.icu import numeric_sort_key as sort_key
 from calibre.utils.img import image_from_data, Canvas, optimize_png, optimize_jpeg
 from calibre.utils.zipfile import ZipFile, ZIP_STORED
 from calibre.utils.filenames import atomic_rename
-from lzma.xz import compress, decompress
+from polyglot.builtins import iteritems, map, range, reraise, filter, as_bytes, unicode_type
+from polyglot import http_client
+from polyglot.queue import Queue, Empty
 
 IMAGE_EXTENSIONS = {'png', 'jpg', 'jpeg'}
 THEME_COVER = 'icon-theme-cover.jpg'
@@ -47,12 +46,14 @@ BASE_URL = 'https://code.calibre-ebook.com/icon-themes/'
 
 COVER_SIZE = (340, 272)
 
+
 def render_svg(filepath):
     must_use_qt(headless=False)
     pngpath = filepath[:-4] + '.png'
     i = QImage(filepath)
     i.save(pngpath)
 
+
 def read_images_from_folder(path):
     name_map = {}
     path = os.path.abspath(path)
@@ -71,12 +72,14 @@ def read_images_from_folder(path):
             name_map[name] = filepath
     return name_map
 
+
 class Theme(object):
 
     def __init__(self, title='', author='', version=-1, description='', license='Unknown', url=None, cover=None):
         self.title, self.author, self.version, self.description = title, author, version, description
         self.license, self.cover, self.url = license, cover, url
 
+
 class Report(object):
 
     def __init__(self, path, name_map, extra, missing, theme):
@@ -87,6 +90,7 @@ class Report(object):
     def name(self):
         return ascii_filename(self.theme.title).replace(' ', '_').replace('.', '_').lower()
 
+
 def read_theme_from_folder(path):
     path = os.path.abspath(path)
     current_image_map = read_images_from_folder(P('images', allow_user_override=False))
@@ -107,6 +111,7 @@ def read_theme_from_folder(path):
     except ValueError:
         # Corrupted metadata file
         metadata = {}
+
     def safe_int(x):
         try:
             return int(x)
@@ -125,6 +130,7 @@ def read_theme_from_folder(path):
         theme.cover = create_cover(ans)
     return ans
 
+
 def icon_for_action(name):
     for plugin in interface_actions():
         if plugin.name == name:
@@ -135,6 +141,7 @@ def icon_for_action(name):
             if icon:
                 return icon
 
+
 def default_cover_icons(cols=5):
     count = 0
     for ac in gprefs.defaults['action-layout-toolbar']:
@@ -152,6 +159,7 @@ def default_cover_icons(cols=5):
         del extra[0]
         count += 1
 
+
 def create_cover(report, icons=(), cols=5, size=120, padding=16):
     icons = icons or tuple(default_cover_icons(cols))
     rows = int(math.ceil(len(icons) / cols))
@@ -171,21 +179,23 @@ def create_cover(report, icons=(), cols=5, size=120, padding=16):
             with lopen(ipath, 'rb') as f:
                 img = image_from_data(f.read())
             scaled, nwidth, nheight = fit_image(img.width(), img.height(), size, size)
-            img = img.scaled(nwidth, nheight, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+            img = img.scaled(nwidth, nheight, Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation)
             dx = (size - nwidth) // 2
             canvas.compose(img, x + dx, y)
     return canvas.export()
 
+
 def verify_theme(report):
     must_use_qt()
     report.bad = bad = {}
-    for name, path in report.name_map.iteritems():
+    for name, path in iteritems(report.name_map):
         reader = QImageReader(os.path.join(report.path, path))
         img = reader.read()
         if img.isNull():
             bad[name] = reader.errorString()
     return bool(bad)
 
+
 class ThemeCreateDialog(Dialog):
 
     def __init__(self, parent, report):
@@ -200,10 +210,10 @@ class ThemeCreateDialog(Dialog):
         self.w = w = QGroupBox(_('Theme Metadata'), self)
         self.splitter.addWidget(w)
         l = w.l = QFormLayout(w)
-        l.setFieldGrowthPolicy(l.ExpandingFieldsGrow)
+        l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.ExpandingFieldsGrow)
         self.missing_icons_group = mg = QGroupBox(self)
         self.mising_icons = mi = QListWidget(mg)
-        mi.setSelectionMode(mi.NoSelection)
+        mi.setSelectionMode(QAbstractItemView.SelectionMode.NoSelection)
         mg.l = QVBoxLayout(mg)
         mg.l.addWidget(mi)
         self.splitter.addWidget(mg)
@@ -223,7 +233,7 @@ class ThemeCreateDialog(Dialog):
             ' Creative Commons or Public Domain.'))
         self.description = QTextEdit(self)
         l.addRow(self.description)
-        self.refresh_button = rb = self.bb.addButton(_('&Refresh'), self.bb.ActionRole)
+        self.refresh_button = rb = self.bb.addButton(_('&Refresh'), QDialogButtonBox.ButtonRole.ActionRole)
         rb.setIcon(QIcon(I('view-refresh.png')))
         rb.clicked.connect(self.refresh)
 
@@ -248,8 +258,11 @@ class ThemeCreateDialog(Dialog):
         }
 
     def save_metadata(self):
+        data = json.dumps(self.metadata, indent=2)
+        if not isinstance(data, bytes):
+            data = data.encode('utf-8')
         with open(os.path.join(self.report.path, THEME_METADATA), 'wb') as f:
-            json.dump(self.metadata, f, indent=2)
+            f.write(data)
 
     def refresh(self):
         self.save_metadata()
@@ -284,6 +297,7 @@ class ThemeCreateDialog(Dialog):
                 'You must specify an author for this icon theme'), show=True)
         return Dialog.accept(self)
 
+
 class Compress(QProgressDialog):
 
     update_signal = pyqtSignal(object, object)
@@ -294,7 +308,7 @@ class Compress(QProgressDialog):
         self.setWindowTitle(self.labelText())
         self.setWindowIcon(QIcon(I('lt.png')))
         self.setMinimumDuration(0)
-        self.update_signal.connect(self.do_update, type=Qt.QueuedConnection)
+        self.update_signal.connect(self.do_update, type=Qt.ConnectionType.QueuedConnection)
         self.raw = self.prefix = None
         self.abort = Event()
         self.canceled.connect(self.abort.set)
@@ -325,6 +339,7 @@ class Compress(QProgressDialog):
         else:
             self.update_signal.emit(self.maximum(), '')
 
+
 def create_themeball(report, progress=None, abort=None):
     pool = ThreadPool(processes=cpu_count())
     buf = BytesIO()
@@ -349,13 +364,13 @@ def create_themeball(report, progress=None, abort=None):
         except Exception:
             return sys.exc_info()
 
-    errors = tuple(filter(None, pool.map(optimize, tuple(report.name_map.iterkeys()))))
+    errors = tuple(filter(None, pool.map(optimize, tuple(report.name_map))))
     pool.close(), pool.join()
     if abort is not None and abort.is_set():
         return
     if errors:
         e = errors[0]
-        raise e[0], e[1], e[2]
+        reraise(*e)
 
     if progress is not None:
         progress(next(num), _('Creating theme file'))
@@ -365,12 +380,12 @@ def create_themeball(report, progress=None, abort=None):
             with lopen(srcpath, 'rb') as f:
                 zf.writestr(name, f.read(), compression=ZIP_STORED)
     buf.seek(0)
-    out = BytesIO()
     if abort is not None and abort.is_set():
         return None, None
     if progress is not None:
         progress(next(num), _('Compressing theme file'))
-    compress(buf, out, level=9)
+    import lzma
+    compressed = lzma.compress(buf.getvalue(), format=lzma.FORMAT_XZ, preset=9)
     buf = BytesIO()
     prefix = report.name
     if abort is not None and abort.is_set():
@@ -379,7 +394,7 @@ def create_themeball(report, progress=None, abort=None):
         with lopen(os.path.join(report.path, THEME_METADATA), 'rb') as f:
             zf.writestr(prefix + '/' + THEME_METADATA, f.read())
         zf.writestr(prefix + '/' + THEME_COVER, create_cover(report))
-        zf.writestr(prefix + '/' + 'icons.zip.xz', out.getvalue(), compression=ZIP_STORED)
+        zf.writestr(prefix + '/' + 'icons.zip.xz', compressed, compression=ZIP_STORED)
     if progress is not None:
         progress(next(num), _('Finished'))
     return buf.getvalue(), prefix
@@ -393,7 +408,7 @@ def create_theme(folder=None, parent=None):
             return
     report = read_theme_from_folder(folder)
     d = ThemeCreateDialog(parent, report)
-    if d.exec_() != d.Accepted:
+    if d.exec_() != QDialog.DialogCode.Accepted:
         return
     d.save_metadata()
     d = Compress(d.report, parent=parent)
@@ -412,6 +427,7 @@ def create_theme(folder=None, parent=None):
 
 # Choose Theme  {{{
 
+
 def download_cover(cover_url, etag=None, cached=b''):
     url = BASE_URL + cover_url
     headers = {}
@@ -425,10 +441,11 @@ def download_cover(cover_url, etag=None, cached=b''):
         etag = response.getheader('ETag', None) or None
         return cached, etag
     except HTTPError as e:
-        if etag and e.code == httplib.NOT_MODIFIED:
+        if etag and e.code == http_client.NOT_MODIFIED:
             return cached, etag
         raise
 
+
 def get_cover(metadata):
     cdir = os.path.join(cache_dir(), 'icon-theme-covers')
     try:
@@ -436,9 +453,11 @@ def get_cover(metadata):
     except EnvironmentError as e:
         if e.errno != errno.EEXIST:
             raise
+
     def path(ext):
         return os.path.join(cdir, metadata['name'] + '.' + ext)
     etag_file, cover_file = map(path, 'etag jpg'.split())
+
     def safe_read(path):
         try:
             with open(path, 'rb') as f:
@@ -448,19 +467,27 @@ def get_cover(metadata):
                 raise
         return b''
     etag, cached = safe_read(etag_file), safe_read(cover_file)
+    etag = etag.decode('utf-8')
     cached, etag = download_cover(metadata['cover-url'], etag, cached)
     if cached:
-        with open(cover_file, 'wb') as f:
+        aname = cover_file + '.atomic'
+        with open(aname, 'wb') as f:
             f.write(cached)
+        atomic_rename(aname, cover_file)
     if etag:
         with open(etag_file, 'wb') as f:
-            f.write(etag)
+            f.write(as_bytes(etag))
     return cached or b''
 
-def get_covers(themes, callback, num_of_workers=8):
+
+def get_covers(themes, dialog, num_of_workers=8):
     items = Queue()
     tuple(map(items.put, themes))
 
+    def callback(metadata, x):
+        if not sip.isdeleted(dialog) and not dialog.dialog_closed:
+            dialog.cover_downloaded.emit(metadata, x)
+
     def run():
         while True:
             try:
@@ -476,11 +503,12 @@ def get_covers(themes, callback, num_of_workers=8):
             else:
                 callback(metadata, cdata)
 
-    for w in xrange(num_of_workers):
+    for w in range(num_of_workers):
         t = Thread(name='IconThemeCover', target=run)
         t.daemon = True
         t.start()
 
+
 class Delegate(QStyledItemDelegate):
 
     SPACING = 10
@@ -489,16 +517,16 @@ class Delegate(QStyledItemDelegate):
         return QSize(COVER_SIZE[0] * 2, COVER_SIZE[1] + 2 * self.SPACING)
 
     def paint(self, painter, option, index):
-        QStyledItemDelegate.paint(self, painter, option, QModelIndex())
-        theme = index.data(Qt.UserRole)
+        QStyledItemDelegate.paint(self, painter, option, empty_index)
+        theme = index.data(Qt.ItemDataRole.UserRole)
         if not theme:
             return
         painter.save()
-        pixmap = index.data(Qt.DecorationRole)
+        pixmap = index.data(Qt.ItemDataRole.DecorationRole)
         if pixmap and not pixmap.isNull():
             rect = option.rect.adjusted(0, self.SPACING, COVER_SIZE[0] - option.rect.width(), - self.SPACING)
             painter.drawPixmap(rect, pixmap)
-        if option.state & QStyle.State_Selected:
+        if option.state & QStyle.StateFlag.State_Selected:
             painter.setPen(QPen(QApplication.instance().palette().highlightedText().color()))
         bottom = option.rect.bottom() - 2
         painter.drawLine(0, bottom, option.rect.right(), bottom)
@@ -517,6 +545,7 @@ class Delegate(QStyledItemDelegate):
         painter.drawStaticText(COVER_SIZE[0] + self.SPACING, option.rect.top() + self.SPACING, theme['static-text'])
         painter.restore()
 
+
 class DownloadProgress(ProgressDialog):
 
     ds = pyqtSignal(object)
@@ -526,9 +555,9 @@ class DownloadProgress(ProgressDialog):
     def __init__(self, parent, size):
         ProgressDialog.__init__(self, _('Downloading icons...'), _(
             'Downloading icons, please wait...'), max=size, parent=parent, icon='download_metadata.png')
-        self.ds.connect(self.bar.setValue, type=Qt.QueuedConnection)
-        self.acc.connect(self.accept, type=Qt.QueuedConnection)
-        self.rej.connect(self.reject, type=Qt.QueuedConnection)
+        self.ds.connect(self.bar.setValue, type=Qt.ConnectionType.QueuedConnection)
+        self.acc.connect(self.accept, type=Qt.ConnectionType.QueuedConnection)
+        self.rej.connect(self.reject, type=Qt.ConnectionType.QueuedConnection)
 
     def downloaded(self, byte_count):
         self.ds.emit(byte_count)
@@ -539,6 +568,7 @@ class DownloadProgress(ProgressDialog):
     def queue_reject(self):
         self.rej.emit()
 
+
 class ChooseTheme(Dialog):
 
     cover_downloaded = pyqtSignal(object, object)
@@ -550,12 +580,17 @@ class ChooseTheme(Dialog):
         except Exception:
             self.current_theme = None
         Dialog.__init__(self, _('Choose an icon theme'), 'choose-icon-theme-dialog', parent)
-        self.themes_downloaded.connect(self.show_themes, type=Qt.QueuedConnection)
-        self.cover_downloaded.connect(self.set_cover, type=Qt.QueuedConnection)
+        self.finished.connect(self.on_finish)
+        self.dialog_closed = False
+        self.themes_downloaded.connect(self.show_themes, type=Qt.ConnectionType.QueuedConnection)
+        self.cover_downloaded.connect(self.set_cover, type=Qt.ConnectionType.QueuedConnection)
         self.keep_downloading = True
         self.commit_changes = None
         self.new_theme_title = None
 
+    def on_finish(self):
+        self.dialog_closed = True
+
     def sizeHint(self):
         desktop  = QApplication.instance().desktop()
         h = desktop.availableGeometry(self).height()
@@ -566,23 +601,25 @@ class ChooseTheme(Dialog):
         self.stack = l = QStackedLayout()
         self.pi = pi = ProgressIndicator(self, 256)
         vl.addLayout(l), vl.addWidget(self.bb)
-        self.restore_defs_button = b = self.bb.addButton(_('Restore &default icons'), self.bb.ActionRole)
+        self.restore_defs_button = b = self.bb.addButton(_('Restore &default icons'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.restore_defaults)
         b.setIcon(QIcon(I('view-refresh.png')))
         self.c = c = QWidget(self)
         self.c.v = v = QVBoxLayout(self.c)
-        v.addStretch(), v.addWidget(pi, 0, Qt.AlignCenter)
+        v.addStretch(), v.addWidget(pi, 0, Qt.AlignmentFlag.AlignCenter)
         self.wait_msg = m = QLabel(self)
-        v.addWidget(m, 0, Qt.AlignCenter), v.addStretch()
-        m.setStyleSheet('QLabel { font-size: 40px; font-weight: bold }')
+        v.addWidget(m, 0, Qt.AlignmentFlag.AlignCenter), v.addStretch()
+        f = m.font()
+        f.setBold(True), f.setPointSize(28), m.setFont(f)
         self.start_spinner()
 
         l.addWidget(c)
         self.w = w = QWidget(self)
         l.addWidget(w)
         w.l = l = QGridLayout(w)
+
         def add_row(x, y=None):
-            if isinstance(x, type('')):
+            if isinstance(x, unicode_type):
                 x = QLabel(x)
             row = l.rowCount()
             if y is None:
@@ -604,7 +641,7 @@ class ChooseTheme(Dialog):
         sb.currentIndexChanged[int].connect(self.re_sort)
         sb.currentIndexChanged[int].connect(lambda : gprefs.set('choose_icon_theme_sort_by', sb.currentIndex()))
         self.theme_list = tl = QListWidget(self)
-        tl.setVerticalScrollMode(tl.ScrollPerPixel)
+        tl.setVerticalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
         self.delegate = Delegate(tl)
         tl.setItemDelegate(self.delegate)
         tl.itemDoubleClicked.connect(self.accept)
@@ -637,9 +674,9 @@ class ChooseTheme(Dialog):
         self.theme_list.clear()
         for theme in self.themes:
             i = QListWidgetItem(theme.get('title', '') + ' %s %s' % (theme.get('number'), self.usage.get(theme.get('name'))), self.theme_list)
-            i.setData(Qt.UserRole, theme)
+            i.setData(Qt.ItemDataRole.UserRole, theme)
             if 'cover-pixmap' in theme:
-                i.setData(Qt.DecorationRole, theme['cover-pixmap'])
+                i.setData(Qt.ItemDataRole.DecorationRole, theme['cover-pixmap'])
 
     def get_themes(self):
 
@@ -662,28 +699,29 @@ class ChooseTheme(Dialog):
             import traceback
             self.themes = traceback.format_exc()
         t.join()
-        self.themes_downloaded.emit()
+        if not sip.isdeleted(self):
+            self.themes_downloaded.emit()
 
     def show_themes(self):
         self.end_spinner()
         if not isinstance(self.themes, list):
             error_dialog(self, _('Failed to download list of themes'), _(
-                'Failed to download list of themes, click "Show Details" for more information'),
+                'Failed to download list of themes, click "Show details" for more information'),
                          det_msg=self.themes, show=True)
             self.reject()
             return
         for theme in self.themes:
             theme['usage'] = self.usage.get(theme['name'], 0)
         self.re_sort()
-        get_covers(self.themes, self.cover_downloaded.emit)
+        get_covers(self.themes, self)
 
     def __iter__(self):
-        for i in xrange(self.theme_list.count()):
+        for i in range(self.theme_list.count()):
             yield self.theme_list.item(i)
 
     def item_from_name(self, name):
         for item in self:
-            if item.data(Qt.UserRole)['name'] == name:
+            if item.data(Qt.ItemDataRole.UserRole)['name'] == name:
                 return item
 
     def set_cover(self, theme, cdata):
@@ -697,7 +735,7 @@ class ChooseTheme(Dialog):
             p.setDevicePixelRatio(dpr)
         item = self.item_from_name(theme['name'])
         if item is not None:
-            item.setData(Qt.DecorationRole, p)
+            item.setData(Qt.ItemDataRole.DecorationRole, p)
 
     def restore_defaults(self):
         if self.current_theme is not None:
@@ -709,10 +747,10 @@ class ChooseTheme(Dialog):
         Dialog.accept(self)
 
     def accept(self):
-        if self.theme_list.currentIndex() < 0:
+        if self.theme_list.currentRow() < 0:
             return error_dialog(self, _('No theme selected'), _(
                 'You must first select an icon theme'), show=True)
-        theme = self.theme_list.currentItem().data(Qt.UserRole)
+        theme = self.theme_list.currentItem().data(Qt.ItemDataRole.UserRole)
         url = BASE_URL + theme['icons-url']
         size = theme['compressed-size']
         theme = {k:theme.get(k, '') for k in 'name title version'.split()}
@@ -745,13 +783,15 @@ class ChooseTheme(Dialog):
 
         if self.downloaded_theme and not isinstance(self.downloaded_theme, BytesIO):
             return error_dialog(self, _('Download failed'), _(
-                'Failed to download icon theme, click "Show Details" for more information.'), show=True, det_msg=self.downloaded_theme)
-        if ret == d.Rejected or not self.keep_downloading or d.canceled or self.downloaded_theme is None:
+                'Failed to download icon theme, click "Show details" for more information.'), show=True, det_msg=self.downloaded_theme)
+        if ret == QDialog.DialogCode.Rejected or not self.keep_downloading or d.canceled or self.downloaded_theme is None:
             return
         dt = self.downloaded_theme
+
         def commit_changes():
+            import lzma
             dt.seek(0)
-            f = decompress(dt)
+            f = BytesIO(lzma.decompress(dt.getvalue()))
             f.seek(0)
             remove_icon_theme()
             install_icon_theme(theme, f)
@@ -761,6 +801,7 @@ class ChooseTheme(Dialog):
 
 # }}}
 
+
 def remove_icon_theme():
     icdir = os.path.join(config_dir, 'resources', 'images')
     metadata_file = os.path.join(icdir, 'icon-theme.json')
@@ -779,12 +820,14 @@ def remove_icon_theme():
                 raise
     os.remove(metadata_file)
 
+
 def safe_copy(src, destpath):
     tpath = destpath + '-temp'
     with open(tpath, 'wb') as dest:
         shutil.copyfileobj(src, dest)
     atomic_rename(tpath, destpath)
 
+
 def install_icon_theme(theme, f):
     icdir = os.path.abspath(os.path.join(config_dir, 'resources', 'images'))
     if not os.path.exists(icdir):
@@ -808,16 +851,16 @@ def install_icon_theme(theme, f):
             theme['files'].add(name)
 
     theme['files'] = tuple(theme['files'])
-    buf = BytesIO()
-    json.dump(theme, buf, indent=2)
+    buf = BytesIO(as_bytes(json.dumps(theme, indent=2)))
     buf.seek(0)
     safe_copy(buf, metadata_file)
 
+
 if __name__ == '__main__':
     from calibre.gui2 import Application
     app = Application([])
     # create_theme('.')
     d = ChooseTheme()
-    if d.exec_() == d.Accepted and d.commit_changes is not None:
+    if d.exec_() == QDialog.DialogCode.Accepted and d.commit_changes is not None:
         d.commit_changes()
     del app
