@@ -7,7 +7,7 @@ __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 from functools import partial
-from PyQt5.Qt import (
+from qt.core import (
     QActionGroup, QCoreApplication, QFrame, QHBoxLayout, QIcon, QLabel, QLineEdit,
     QMenu, QObject, QSizePolicy, Qt, QToolButton, QVBoxLayout, QWidget, pyqtSignal
 )
@@ -361,7 +361,7 @@ class MainWindowMixin(object):  # {{{
             try:
                 self.setUnifiedTitleAndToolBarOnMac(True)
             except AttributeError:
-                pass  # PyQt5 seems to be missing this property
+                pass  # PyQt seems to be missing this property
 
         # And now, start adding the real widgets
         l.addWidget(self.search_bar)
