@@ -1,19 +1,21 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import textwrap
 
-from PyQt5.Qt import (QWidget, pyqtSignal, QCheckBox, QAbstractSpinBox,
+from qt.core import (QWidget, pyqtSignal, QCheckBox, QAbstractSpinBox, QApplication,
     QLineEdit, QComboBox, Qt, QIcon, QDialog, QVBoxLayout,
     QDialogButtonBox)
 
 from calibre.customize.ui import preferences_plugins
 from calibre.utils.config import ConfigProxy
 from calibre.gui2.complete2 import EditWithComplete
+from calibre.gui2.widgets import HistoryLineEdit
 from polyglot.builtins import unicode_type, string_or_bytes
 
 
@@ -21,6 +23,10 @@ class AbortCommit(Exception):
     pass
 
 
+class AbortInitialize(Exception):
+    pass
+
+
 class ConfigWidgetInterface(object):
 
     '''
@@ -57,7 +63,8 @@ class ConfigWidgetInterface(object):
     def initialize(self):
         '''
         Should set all config values to their initial values (the values
-        stored in the config files).
+        stored in the config files). A "return" statement is optional. Return
+        False if the dialog is not to be shown.
         '''
         raise NotImplementedError()
 
@@ -88,7 +95,7 @@ class ConfigWidgetInterface(object):
 
 class Setting(object):
 
-    CHOICES_SEARCH_FLAGS = Qt.MatchExactly | Qt.MatchCaseSensitive
+    CHOICES_SEARCH_FLAGS = Qt.MatchFlag.MatchExactly | Qt.MatchFlag.MatchCaseSensitive
 
     def __init__(self, name, config_obj, widget, gui_name=None,
             empty_string_is_None=True, choices=None, restart_required=False):
@@ -108,9 +115,11 @@ class Setting(object):
         elif isinstance(self.gui_obj, QAbstractSpinBox):
             self.datatype = 'number'
             self.gui_obj.valueChanged.connect(self.changed)
-        elif isinstance(self.gui_obj, QLineEdit):
+        elif isinstance(self.gui_obj, (QLineEdit, HistoryLineEdit)):
             self.datatype = 'string'
             self.gui_obj.textChanged.connect(self.changed)
+            if isinstance(self.gui_obj, HistoryLineEdit):
+                self.gui_obj.initialize('preferences_setting_' + self.name)
         elif isinstance(self.gui_obj, QComboBox):
             self.datatype = 'choice'
             self.gui_obj.editTextChanged.connect(self.changed)
@@ -183,7 +192,7 @@ class Setting(object):
             if isinstance(self.gui_obj, EditWithComplete):
                 self.gui_obj.setText(val)
             else:
-                idx = self.gui_obj.findData((val), role=Qt.UserRole,
+                idx = self.gui_obj.findData((val), role=Qt.ItemDataRole.UserRole,
                         flags=self.CHOICES_SEARCH_FLAGS)
                 if idx == -1:
                     idx = 0
@@ -214,7 +223,7 @@ class CommaSeparatedList(Setting):
     def set_gui_val(self, val):
         x = ''
         if val:
-            x = u', '.join(val)
+            x = ', '.join(val)
         self.gui_obj.setText(x)
 
     def get_gui_val(self):
@@ -353,23 +362,23 @@ def show_config_widget(category, name, gui=None, show_restart_msg=False,
     d.setWindowTitle(_('Configure ') + pl.gui_name)
     d.setWindowIcon(QIcon(I('config.png')))
     bb = QDialogButtonBox(d)
-    bb.setStandardButtons(bb.Apply|bb.Cancel|bb.RestoreDefaults)
+    bb.setStandardButtons(QDialogButtonBox.StandardButton.Apply|QDialogButtonBox.StandardButton.Cancel|QDialogButtonBox.StandardButton.RestoreDefaults)
     bb.accepted.connect(d.accept)
     bb.rejected.connect(d.reject)
     w = pl.create_widget(d)
     d.set_widget(w)
-    bb.button(bb.RestoreDefaults).clicked.connect(w.restore_defaults)
-    bb.button(bb.RestoreDefaults).setEnabled(w.supports_restoring_to_defaults)
-    bb.button(bb.Apply).setEnabled(False)
-    bb.button(bb.Apply).clicked.connect(d.accept)
+    bb.button(QDialogButtonBox.StandardButton.RestoreDefaults).clicked.connect(w.restore_defaults)
+    bb.button(QDialogButtonBox.StandardButton.RestoreDefaults).setEnabled(w.supports_restoring_to_defaults)
+    bb.button(QDialogButtonBox.StandardButton.Apply).setEnabled(False)
+    bb.button(QDialogButtonBox.StandardButton.Apply).clicked.connect(d.accept)
 
     def onchange():
-        b = bb.button(bb.Apply)
+        b = bb.button(QDialogButtonBox.StandardButton.Apply)
         b.setEnabled(True)
         b.setDefault(True)
         b.setAutoDefault(True)
     w.changed_signal.connect(onchange)
-    bb.button(bb.Cancel).setFocus(True)
+    bb.button(QDialogButtonBox.StandardButton.Cancel).setFocus(True)
     l = QVBoxLayout()
     d.setLayout(l)
     l.addWidget(w)
@@ -381,7 +390,7 @@ def show_config_widget(category, name, gui=None, show_restart_msg=False,
     w.genesis(gui)
     w.initialize()
     if geom is not None:
-        d.restoreGeometry(geom)
+        QApplication.instance().safe_restore_geometry(d, geom)
     d.exec_()
     geom = bytearray(d.saveGeometry())
     gprefs[conf_name] = geom
@@ -401,7 +410,7 @@ def test_widget(category, name, gui=None):
 
 
 def test_all():
-    from PyQt5.Qt import QApplication
+    from qt.core import QApplication
     app = QApplication([])
     app
     gui = init_gui()
