@@ -6,26 +6,34 @@ __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, textwrap, json
+import json
+import os
+import textwrap
+from functools import partial
+from qt.core import (
+    QAbstractItemView, QAbstractListModel, QApplication, QCheckBox, QComboBox,
+    QDialog, QDialogButtonBox, QDoubleValidator, QFrame, QGridLayout, QIcon,
+    QIntValidator, QItemSelectionModel, QLabel, QLineEdit, QListView, QMenu,
+    QPalette, QPushButton, QScrollArea, QSize, QSizePolicy, QSpacerItem,
+    QStandardItem, QStandardItemModel, Qt, QToolButton, QVBoxLayout, QWidget,
+    QItemSelection, pyqtSignal
+)
 
-from qt.core import (QWidget, QDialog, QLabel, QGridLayout, QComboBox, QSize,
-        QLineEdit, QIntValidator, QDoubleValidator, QFrame, Qt, QIcon, QHBoxLayout,
-        QScrollArea, QPushButton, QVBoxLayout, QDialogButtonBox, QToolButton, QItemSelectionModel,
-        QListView, QAbstractListModel, pyqtSignal, QSizePolicy, QSpacerItem, QPalette,
-        QApplication, QStandardItem, QStandardItemModel, QCheckBox, QMenu, QAbstractItemView)
-
-from calibre import prepare_string_for_xml, sanitize_file_name, as_unicode
+from calibre import as_unicode, prepare_string_for_xml, sanitize_file_name
 from calibre.constants import config_dir
-from calibre.utils.icu import sort_key
-from calibre.gui2 import (error_dialog, choose_files, pixmap_to_data, gprefs,
-                          choose_save_file, open_local_file)
+from calibre.gui2 import (
+    choose_files, choose_save_file, error_dialog, gprefs, open_local_file,
+    pixmap_to_data
+)
 from calibre.gui2.dialogs.template_dialog import TemplateDialog
 from calibre.gui2.metadata.single_download import RichTextDelegate
-from calibre.gui2.widgets2 import ColorButton
-from calibre.library.coloring import (Rule, conditionable_columns,
-    displayable_columns, rule_from_template, color_row_key)
+from calibre.gui2.widgets2 import ColorButton, FlowLayout, Separator
+from calibre.library.coloring import (
+    Rule, color_row_key, conditionable_columns, displayable_columns,
+    rule_from_template
+)
+from calibre.utils.icu import lower, sort_key
 from calibre.utils.localization import lang_map
-from calibre.utils.icu import lower
 from polyglot.builtins import iteritems, unicode_type
 
 all_columns_string = _('All columns')
@@ -935,12 +943,12 @@ class EditRules(QWidget):  # {{{
         self.up_button = b = QToolButton(self)
         b.setIcon(QIcon(I('arrow-up.png')))
         b.setToolTip(_('Move the selected rule up'))
-        b.clicked.connect(self.move_up)
+        b.clicked.connect(partial(self.move_rows, moving_up=True))
         g.addWidget(b, 0, 1, 1, 1, Qt.AlignmentFlag.AlignTop)
         self.down_button = b = QToolButton(self)
         b.setIcon(QIcon(I('arrow-down.png')))
         b.setToolTip(_('Move the selected rule down'))
-        b.clicked.connect(self.move_down)
+        b.clicked.connect(partial(self.move_rows, moving_up=False))
         g.addWidget(b, 1, 1, 1, 1, Qt.AlignmentFlag.AlignBottom)
 
         l.addLayout(g, l.rowCount(), 0, 1, 2)
@@ -949,7 +957,7 @@ class EditRules(QWidget):  # {{{
         self.add_advanced_button = b = QPushButton(QIcon(I('plus.png')),
                 _('Add ad&vanced rule'), self)
         b.clicked.connect(self.add_advanced)
-        self.hb = hb = QHBoxLayout()
+        self.hb = hb = FlowLayout()
         l.addLayout(hb, l.rowCount(), 0, 1, 2)
         hb.addWidget(b)
         self.duplicate_rule_button = b = QPushButton(QIcon(I('edit-copy.png')),
@@ -962,13 +970,15 @@ class EditRules(QWidget):  # {{{
         b.clicked.connect(self.convert_to_advanced)
         b.setEnabled(False)
         hb.addWidget(b)
-        hb.addStretch(10)
+        sep = Separator(self, b)
+        hb.addWidget(sep)
+
         self.open_icon_folder_button = b = QPushButton(QIcon(I('icon_choose.png')),
                 _('Open icon folder'), self)
-        connect_lambda(b.clicked, self,
-                       lambda _: open_local_file(os.path.join(config_dir, 'cc_icons')))
+        b.clicked.connect(self.open_icon_folder)
         hb.addWidget(b)
-        hb.addStretch(10)
+        sep = Separator(self, b)
+        hb.addWidget(sep)
         self.export_button = b = QPushButton(_('E&xport'), self)
         b.clicked.connect(self.export_rules)
         b.setToolTip(_('Export these rules to a file'))
@@ -978,6 +988,11 @@ class EditRules(QWidget):  # {{{
         b.clicked.connect(self.import_rules)
         hb.addWidget(b)
 
+    def open_icon_folder(self):
+        path = os.path.join(config_dir, 'cc_icons')
+        os.makedirs(path, exist_ok=True)
+        open_local_file(path)
+
     def initialize(self, fm, prefs, mi, pref_name):
         self.pref_name = pref_name
         self.model = RulesModel(prefs, fm, self.pref_name)
@@ -1132,34 +1147,26 @@ class EditRules(QWidget):  # {{{
                 self.model.remove_rule(row)
             self.changed.emit()
 
-    def move_up(self):
+    def move_rows(self, moving_up=True):
         sm = self.rules_view.selectionModel()
-        rows = sorted(list(sm.selectedRows()))
+        rows = sorted(list(sm.selectedRows()), reverse=not moving_up)
         if rows:
-            if rows[0].row() == 0:
+            if rows[0].row() == (0 if moving_up else self.model.rowCount() - 1):
                 return
             sm.clear()
+            indices_to_select = []
             for idx in rows:
                 if idx.isValid():
-                    idx = self.model.move(idx, -1)
+                    idx = self.model.move(idx, -1 if moving_up else 1)
                     if idx is not None:
-                        sm.select(idx, QItemSelectionModel.SelectionFlag.Toggle)
-                        self.rules_view.setCurrentIndex(idx)
-            self.changed.emit()
-
-    def move_down(self):
-        sm = self.rules_view.selectionModel()
-        rows = sorted(list(sm.selectedRows()))
-        if rows:
-            if rows[-1].row() == self.model.rowCount() - 1:
-                return
-            sm.clear()
-            for idx in rows:
-                if idx.isValid():
-                    idx = self.model.move(idx, 1)
-                    if idx is not None:
-                        sm.select(idx, QItemSelectionModel.SelectionFlag.Toggle)
-                        self.rules_view.setCurrentIndex(idx)
+                        indices_to_select.append(idx)
+            if indices_to_select:
+                new_selections = QItemSelection()
+                for idx in indices_to_select:
+                    new_selections.merge(QItemSelection(idx, idx),
+                                         QItemSelectionModel.SelectionFlag.Select)
+                sm.select(new_selections, QItemSelectionModel.SelectionFlag.Select)
+                self.rules_view.scrollTo(indices_to_select[0])
             self.changed.emit()
 
     def clear(self):
