@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -12,13 +12,14 @@ from calibre.constants import DEBUG
 from calibre.db.errors import NoSuchFormat
 from calibre.utils.config import Config, StringConfig, tweaks
 from calibre.utils.formatter import TemplateFormatter
-from calibre.utils.filenames import shorten_components_to, supports_long_names, ascii_filename
+from calibre.utils.filenames import shorten_components_to, ascii_filename
 from calibre.constants import preferred_encoding
 from calibre.ebooks.metadata import fmt_sidx
 from calibre.ebooks.metadata import title_sort
 from calibre.utils.date import as_local_time
-from calibre import strftime, prints, sanitize_file_name_unicode
+from calibre import strftime, prints, sanitize_file_name
 from calibre.db.lazy import FormatsList
+from polyglot.builtins import unicode_type
 
 plugboard_any_device_value = 'any device'
 plugboard_any_format_value = 'any format'
@@ -93,24 +94,22 @@ def config(defaults=None):
             help=_('Comma separated list of formats to save for each book.'
                 ' By default all available formats are saved.'))
     x('template', default=DEFAULT_TEMPLATE,
-            help=_('The template to control the filename and directory structure of the saved files. '
+            help=_('The template to control the filename and folder structure of the saved files. '
                 'Default is "%(templ)s" which will save books into a per-author '
-                'subdirectory with filenames containing title and author. '
+                'subfolder with filenames containing title and author. '
                 'Available controls are: {%(controls)s}')%dict(
                     templ=DEFAULT_TEMPLATE, controls=', '.join(sorted(FORMAT_ARGS))))
     x('send_template', default=DEFAULT_SEND_TEMPLATE,
-            help=_('The template to control the filename and directory structure of files '
+            help=_('The template to control the filename and folder structure of files '
                 'sent to the device. '
                 'Default is "%(templ)s" which will save books into a per-author '
-                'directory with filenames containing title and author. '
+                'folder with filenames containing title and author. '
                 'Available controls are: {%(controls)s}')%dict(
                     templ=DEFAULT_SEND_TEMPLATE, controls=', '.join(FORMAT_ARGS)))
-    x('asciiize', default=True,
-            help=_('Normally, calibre will convert all non English characters into English equivalents '
-                'for the file names. '
-                'WARNING: If you turn this off, you may experience errors when '
-                'saving, depending on how well the filesystem you are saving '
-                'to supports unicode.'))
+    x('asciiize', default=False,
+            help=_('Have calibre convert all non English characters into English equivalents '
+                'for the file names. This is useful if saving to a legacy filesystem '
+                'without full support for Unicode filenames.'))
     x('timefmt', default='%b, %Y',
             help=_('The format in which to display dates. %(day)s - day,'
                 ' %(month)s - month, %(mn)s - month number, %(year)s - year. Default is: %(default)s'
@@ -124,8 +123,8 @@ def config(defaults=None):
     x('replace_whitespace', default=False,
             help=_('Replace whitespace with underscores.'))
     x('single_dir', default=False,
-            help=_('Save into a single directory, ignoring the template'
-                ' directory structure'))
+            help=_('Save into a single folder, ignoring the template'
+                ' folder structure'))
     return c
 
 
@@ -133,7 +132,7 @@ def preprocess_template(template):
     template = template.replace('//', '/')
     template = template.replace('{author}', '{authors}')
     template = template.replace('{tag}', '{tags}')
-    if not isinstance(template, unicode):
+    if not isinstance(template, unicode_type):
         template = template.decode(preferred_encoding, 'replace')
     return template
 
@@ -158,7 +157,7 @@ class Formatter(TemplateFormatter):
                     return self.composite_values[key]
                 self.composite_values[key] = 'RECURSIVE_COMPOSITE FIELD (S2D) ' + key
                 self.composite_values[key] = \
-                    self.evaluate(b['display']['composite_template'], [], kwargs)
+                    self.evaluate(b['display']['composite_template'], [], kwargs, {})
                 return self.composite_values[key]
             if key in kwargs:
                 val = kwargs[key]
@@ -216,7 +215,7 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
     if hasattr(mi, 'last_modified') and hasattr(mi.last_modified, 'timetuple'):
         format_args['last_modified'] = strftime(timefmt, mi.last_modified.timetuple())
 
-    format_args['id'] = str(id)
+    format_args['id'] = unicode_type(id)
     # Now format the custom fields
     custom_metadata = mi.get_all_user_metadata(make_copy=False)
     for key in custom_metadata:
@@ -235,7 +234,7 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
                         divide_by=2.0)
             elif cm['datatype'] in ['int', 'float']:
                 if format_args[key] != 0:
-                    format_args[key] = unicode(format_args[key])
+                    format_args[key] = unicode_type(format_args[key])
                 else:
                     format_args[key] = ''
     if safe_format:
@@ -246,7 +245,7 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
     components = [x.strip() for x in components.split('/')]
     components = [sanitize_func(x) for x in components if x]
     if not components:
-        components = [str(id)]
+        components = [unicode_type(id)]
     if to_lowercase:
         components = [x.lower() for x in components]
     if replace_whitespace:
@@ -278,11 +277,11 @@ def save_book_to_disk(book_id, db, root, opts, length):
 def get_path_components(opts, mi, book_id, path_length):
     try:
         components = get_components(opts.template, mi, book_id, opts.timefmt, path_length,
-            ascii_filename if opts.asciiize else sanitize_file_name_unicode,
+            ascii_filename if opts.asciiize else sanitize_file_name,
             to_lowercase=opts.to_lowercase,
             replace_whitespace=opts.replace_whitespace, safe_format=False,
             last_has_extension=False, single_dir=opts.single_dir)
-    except Exception, e:
+    except Exception as e:
         raise ValueError(_('Failed to calculate path for '
             'save to disk. Template: %(templ)s\n'
             'Error: %(err)s')%dict(templ=opts.template, err=e))
@@ -359,7 +358,7 @@ def do_save_book_to_disk(db, book_id, mi, plugboards,
         return not formats_written, book_id, mi.title
 
     for fmt in formats:
-        fmt_path = base_path+'.'+str(fmt)
+        fmt_path = base_path+'.'+unicode_type(fmt)
         try:
             db.copy_format_to(book_id, fmt, fmt_path)
             formats_written = True
@@ -378,7 +377,7 @@ def sanitize_args(root, opts):
     root = os.path.abspath(root)
 
     opts.template = preprocess_template(opts.template)
-    length = 1000 if supports_long_names(root) else 240
+    length = 240
     length -= len(root)
     if length < 5:
         raise ValueError('%r is too long.'%root)
@@ -436,18 +435,17 @@ def update_serialized_metadata(book, common_data=None):
     plugboard_cache = common_data
     from calibre.customize.ui import apply_null_metadata
     with apply_null_metadata:
+        fmts = [fp.rpartition(os.extsep)[-1] for fp in book['fmts']]
+        mi, cdata = read_serialized_metadata(book)
 
-            fmts = [fp.rpartition(os.extsep)[-1] for fp in book['fmts']]
-            mi, cdata = read_serialized_metadata(book)
+        def report_error(fmt, tb):
+            result.append((fmt, tb))
 
-            def report_error(fmt, tb):
-                result.append((fmt, tb))
-
-            for fmt, fmtpath in zip(fmts, book['fmts']):
-                try:
-                    with lopen(fmtpath, 'r+b') as stream:
-                        update_metadata(mi, fmt, stream, (), cdata, error_report=report_error, plugboard_cache=plugboard_cache)
-                except Exception:
-                    report_error(fmt, traceback.format_exc())
+        for fmt, fmtpath in zip(fmts, book['fmts']):
+            try:
+                with lopen(fmtpath, 'r+b') as stream:
+                    update_metadata(mi, fmt, stream, (), cdata, error_report=report_error, plugboard_cache=plugboard_cache)
+            except Exception:
+                report_error(fmt, traceback.format_exc())
 
     return result
