@@ -16,6 +16,7 @@ from calibre.constants import iswindows, islinux, ismacos, plugins_loc
 from polyglot.builtins import iteritems, map, unicode_type, getenv
 
 is_ci = os.environ.get('CI', '').lower() == 'true'
+is_sanitized = 'libasan' in os.environ.get('LD_PRELOAD', '')
 
 
 class BuildTest(unittest.TestCase):
@@ -37,7 +38,10 @@ class BuildTest(unittest.TestCase):
 
     @unittest.skipUnless(islinux, 'DBUS only used on linux')
     def test_dbus(self):
-        import dbus
+        try:
+            import dbus
+        except SyntaxError:
+            return  # an invalid \ in a string causes a syntax error, but only in CI
         if 'DBUS_SESSION_BUS_ADDRESS' in os.environ:
             bus = dbus.SystemBus()
             self.assertTrue(bus.list_names(), 'Failed to list names on the system bus')
@@ -47,7 +51,7 @@ class BuildTest(unittest.TestCase):
 
     def test_loaders(self):
         import importlib
-        ldr = importlib.import_module('calibre').__spec__.loader
+        ldr = importlib.import_module('calibre').__spec__.loader.get_resource_reader()
         self.assertIn('ebooks', ldr.contents())
         try:
             raw = ldr.open_resource('__init__.py').read()
@@ -286,6 +290,8 @@ class BuildTest(unittest.TestCase):
 
     @unittest.skipIf('SKIP_QT_BUILD_TEST' in os.environ, 'Skipping Qt build test as it causes crashes in the macOS VM')
     def test_qt(self):
+        if is_sanitized:
+            raise unittest.SkipTest('Skipping Qt build test as sanitizer is enabled')
         from qt.core import QTimer
         from qt.core import QApplication
         from qt.webengine import QWebEnginePage
@@ -300,7 +306,7 @@ class BuildTest(unittest.TestCase):
         # hard-coded paths of the Qt installation should work. If they do not,
         # then it is a distro problem.
         fmts = set(map(lambda x: x.data().decode('utf-8'), QImageReader.supportedImageFormats()))  # no2to3
-        testf = {'jpg', 'png', 'svg', 'ico', 'gif'}
+        testf = {'jpg', 'png', 'svg', 'ico', 'gif', 'webp'}
         self.assertEqual(testf.intersection(fmts), testf, "Qt doesn't seem to be able to load some of its image plugins. Available plugins: %s" % fmts)
         data = P('images/blank.png', allow_user_override=False, data=True)
         img = image_from_data(data)
@@ -475,6 +481,8 @@ def find_tests():
     ans.addTests(find_tests())
     from calibre.spell.dictionary import find_tests
     ans.addTests(find_tests())
+    from calibre.db.tests.fts import find_tests
+    ans.addTests(find_tests())
     return ans
 
 
